#!/usr/bin/python

#################################################################
#
#	Convert n dimensional coordinates into a Z value in space
#	filling curve, and vice versa
#
#################################################################

import sys, string

def __apply1__(list, func1):
	newlist = []

	for i in range(len(list)):
		newlist.append(func1(list[i]))

	return newlist

########################################
#	Encoding
########################################
def cord_to_z(list):
	#	Z space filling curve algorithm 
	#	corrdinates to z

	#	find out the number of resolution
	result = 0

	#	format: resol <coord 1> <coord 2> <coord 3> ...
	resultBitCount = list[0] * len(list[1:]) - 1
	for j in range(list[0]-1, -1, -1):
		for i in range(len(list[1:])):
			if ((1 << j) & list[i+1]):
				result = result | (1 << resultBitCount)
			resultBitCount = resultBitCount - 1 

	return result

########################################			
#	Decode
########################################	
def z_to_coord(list):
	#	Z space filling curve algorithm 
	#	z index numbers to corrdinates

	#	format: resol <num of dim> <z index value>
	resultBitCount = list[0] * list[1] - 1
	corrdinates = [0] * list[1]

	for j in range(list[0]-1, -1, -1):
		for i in range(list[1]):
			if ((1 << resultBitCount) & list[2]):
				corrdinates[i] = corrdinates[i] | (1 << j)
			resultBitCount = resultBitCount - 1

	return corrdinates
			

def	argv_2_coord(list):
	#	convert standard arguments to coordinates

	return  __apply1__(list, string.atoi)
	
if __name__ == '__main__':
	values_list = argv_2_coord(sys.argv[2:])

	if (not len(values_list) or len(sys.argv) == 3):
		print 'Wrong arguments'
	else:
		#########################################
		#	Perform Encoding
		#########################################
		if (sys.argv[1] == 'e'):
			result = cord_to_z(values_list)
			print 'Coord list: %s and the Z value is %d' % (values_list[1:], result)

		#########################################
		#	Perform Decoding
		#########################################
		elif (sys.argv[1] == 'd'):
			result = z_to_coord(values_list)
			print 'Z index values: %d and the coordinates are %s' % (values_list[-1], result)

		########################################
		#	Testing routine
		########################################
		elif (sys.argv[1] == 't'):
			resol, dim = values_list[0], values_list[1]
			for k in range(2**((resol-1)* dim)):
				z_arg = [resol, dim, k]
				coordinates = z_to_coord(z_arg)
				coordinates[0:0] = [resol]
				result = cord_to_z(coordinates)
				if (result != k):
					print 'Problem in converting z index value %d != %d' % (result, k)
					sys.exit()
				else:
					print 'Fine in %d' % result
		else:
			print 'Unrecognize mode'

